package cc.fascinated.backend.common;

import lombok.experimental.UtilityClass;

@UtilityClass
public class MathUtils {

    /**
     * Clamps a value between a minimum and maximum.
     *
     * @param value The value to clamp.
     * @param min The minimum value.
     * @param max The maximum value.
     * @return The clamped value.
     */
    public static double clamp(double value, double min, double max) {
        return Math.max(min, Math.min(max, value));
    }

    /**
     * Linearly interpolates between two values.
     *
     * @param a The first value.
     * @param b The second value.
     * @param t The interpolation value.
     * @return The interpolated value.
     */
    public static double lerp(double a, double b, double t) {
        return a + t * (b - a);
    }
}
